package de.intevation.flys.exports;

import java.io.OutputStream;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import au.com.bytecode.opencsv.CSVWriter;

import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.MiddleBedHeightData;
import de.intevation.flys.utils.FLYSUtils;
import de.intevation.flys.utils.Formatter;


/**
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class MiddleBedHeightExporter extends AbstractExporter {

    /** Private logger. */
    private static final Logger logger =
        Logger.getLogger(MiddleBedHeightExporter.class);

    public static final String CSV_KM =
        "export.bedheight_middle.csv.header.km";

    public static final String CSV_SOUNDING =
        "export.bedheight_middle.csv.header.sounding";

    public static final String CSV_HEIGHT =
        "export.bedheight_middle.csv.header.height";

    public static final String CSV_UNCERTAINTY =
        "export.bedheight_middle.csv.header.uncertainty";

    public static final String CSV_DATA_GAP =
        "export.bedheight_middle.csv.header.datagap";

    public static final String CSV_SOUNDING_WIDTH =
        "export.bedheight_middle.csv.header.soundingwidth";

    public static final String CSV_WIDTH =
        "export.bedheight_middle.csv.header.width";

    public static final String CSV_LOCATIONS =
        "export.bedheight_middle.csv.header.locations";


    protected List<MiddleBedHeightData[]> data;


    public void init(Document request, OutputStream out, CallContext cc) {
        super.init(request, out, cc);
        data = new ArrayList<MiddleBedHeightData[]>();
    }


    @Override
    protected void addData(Object d) {
        if (d instanceof CalculationResult) {
            d = ((CalculationResult) d).getData();

            if (d instanceof MiddleBedHeightData[]) {
                logger.debug("Add new data of type MiddleBedHeightData");
                data.add((MiddleBedHeightData[]) d);
            }
        }
    }


    @Override
    protected void writeCSVData(CSVWriter writer) {
        logger.info("MiddleBedHeightExporter.writeCSVData");
        logger.debug("CSV gets " + data.size() + " MiddleBedHeightData objects.");

        writeCSVHeader(writer);

        for (MiddleBedHeightData[] d: data) {
            data2CSV(writer, d);
        }
    }


    protected void writeCSVHeader(CSVWriter writer) {
        writer.writeNext(new String[] {
            msg(CSV_KM, CSV_KM),
            msg(CSV_SOUNDING, CSV_SOUNDING),
            msg(CSV_HEIGHT, CSV_HEIGHT),
            msg(CSV_UNCERTAINTY, CSV_UNCERTAINTY),
            msg(CSV_DATA_GAP, CSV_DATA_GAP),
            msg(CSV_SOUNDING_WIDTH, CSV_SOUNDING_WIDTH),
            msg(CSV_WIDTH, CSV_WIDTH),
            msg(CSV_LOCATIONS, CSV_LOCATIONS)
        });
    }


    protected void data2CSV(CSVWriter writer, MiddleBedHeightData[] mData) {
        logger.debug("Add next MiddleBedHeightData to CSV");

        FLYSArtifact flys = (FLYSArtifact) master;

        NumberFormat kmF     = Formatter.getMiddleBedHeightKM(context);
        NumberFormat heightF = Formatter.getMiddleBedHeightHeight(context);
        NumberFormat uncertF = Formatter.getMiddleBedHeightUncert(context);
        NumberFormat gapF    = Formatter.getMiddleBedHeightDataGap(context);
        NumberFormat soundF  = Formatter.getMiddleBedHeightSounding(context);
        NumberFormat widthF  = Formatter.getMiddleBedHeightWidth(context);

        for (MiddleBedHeightData data: mData) {
            for (int i = 0, n = data.size(); i < n; i++) {
                int start = data.getStartYear();
                int end   = data.getEndYear();

                if (start == end) {
                    writer.writeNext(new String[] {
                        kmF.format(data.getKM(i)),
                        data.getSoundingName(context),
                        heightF.format(data.getMiddleHeight(i)),
                        uncertF.format(data.getUncertainty(i)),
                        gapF.format(data.getDataGap(i)) + "%",
                        soundF.format(data.getSoundingWidth(i)),
                        widthF.format(data.getWidth(i)),
                        FLYSUtils.getLocationDescription(flys, data.getKM(i)),
                    });
                }
                else {
                    writer.writeNext(new String[] {
                        kmF.format(data.getKM(i)),
                        data.getSoundingName(context),
                        heightF.format(data.getMiddleHeight(i)),
                        "",
                        "",
                        "",
                        "",
                        FLYSUtils.getLocationDescription(flys, data.getKM(i)),
                    });
                }

            }
        }
    }


    @Override
    protected void writePDF(OutputStream out) {
        logger.error("TODO: Implement FlowVelocityExporter.writePDF");
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
